@@ -1,10 +1,9 @@
 from .preprocessor import *
+from .backbones import *
 from .detection import *
 from .recognition import *
 from . import utils
 from ._utils import *
-from .vgg import *
 from .core import *
 from .export import *
 from .zoo import *
-from .resnet import *
