@@ -4,21 +4,42 @@
 # See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
 
 import numpy as np
-from rapidfuzz.string_metric import levenshtein
-from typing import List, Tuple
+from typing import List, Tuple, Dict
+from unidecode import unidecode
 from scipy.optimize import linear_sum_assignment
 
-__all__ = ['ExactMatch', 'box_iou', 'assign_pairs', 'LocalizationConfusion', 'OCRMetric']
+__all__ = ['TextMatch', 'box_iou', 'LocalizationConfusion', 'OCRMetric']
 
 
-class ExactMatch:
-    """Implements exact match metric (word-level accuracy) for recognition task.
+def string_match(word1: str, word2: str) -> Tuple[bool, bool, bool, bool]:
+    """Perform string comparison with multiple levels of tolerance
 
-    The aggregated metric is computed as follows:
+    Args:
+        word1: a string
+        word2: another string
+
+    Returns:
+        a tuple with booleans specifying respectively whether the raw strings, their lower-case counterparts, their
+            unidecode counterparts and their lower-case unidecode counterparts match
+    """
+    raw_match = (word1 == word2)
+    caseless_match = (word1.lower() == word2.lower())
+    unidecode_match = (unidecode(word1) == unidecode(word2))
+
+    # Warning: the order is important here otherwise the pair ("EUR", "€") cannot be matched
+    unicase_match = (unidecode(word1).lower() == unidecode(word2).lower())
+
+    return raw_match, caseless_match, unidecode_match, unicase_match
+
+
+class TextMatch:
+    """Implements text match metric (word-level accuracy) for recognition task.
+
+    The raw aggregated metric is computed as follows:
 
     .. math::
         \\forall X, Y \\in \\mathcal{W}^N,
-        ExactMatch(X, Y) = \\frac{1}{N} \\sum\\limits_{i=1}^N f_{Y_i}(X_i)
+        TextMatch(X, Y) = \\frac{1}{N} \\sum\\limits_{i=1}^N f_{Y_i}(X_i)
 
     with the indicator function :math:`f_{a}` defined as:
 
@@ -35,37 +56,14 @@ class ExactMatch:
     :math:`N` is a strictly positive integer.
 
     Example::
-        >>> from doctr.utils import ExactMatch
-        >>> metric = ExactMatch()
+        >>> from doctr.utils import TextMatch
+        >>> metric = TextMatch()
         >>> metric.update(['Hello', 'world'], ['hello', 'world'])
         >>> metric.summary()
+    """
 
-    Args:
-        ignore_case: if true, ignore letter case when computing metric
-        ignore_accents: if true, ignore accents errors when computing metrics"""
-
-    def __init__(
-        self,
-        ignore_case: bool = False,
-        ignore_accents: bool = False,
-    ) -> None:
-
-        self.matches = 0
-        self.total = 0
-        self.ignore_case = ignore_case
-        self.ignore_accents = ignore_accents
-
-    @staticmethod
-    def remove_accent(input_string: str) -> str:
-        """Removes all accents (¨^çéè...) from input_string
-
-        Args:
-            input_string: character sequence with accents
-
-        Returns:
-            character sequence without accents"""
-
-        raise NotImplementedError
+    def __init__(self) -> None:
+        self.reset()
 
     def update(
         self,
@@ -81,31 +79,37 @@ class ExactMatch:
         if len(gt) != len(pred):
             raise AssertionError("prediction size does not match with ground-truth labels size")
 
-        for pred_word, gt_word in zip(pred, gt):
-            if self.ignore_accents:
-                gt_word = self.remove_accent(gt_word)
-                pred_word = self.remove_accent(pred_word)
-
-            if self.ignore_case:
-                gt_word = gt_word.lower()
-                pred_word = pred_word.lower()
-
-            if pred_word == gt_word:
-                self.matches += 1
+        for gt_word, pred_word in zip(gt, pred):
+            _raw, _caseless, _unidecode, _unicase = string_match(gt_word, pred_word)
+            self.raw += int(_raw)
+            self.caseless += int(_caseless)
+            self.unidecode += int(_unidecode)
+            self.unicase += int(_unicase)
 
         self.total += len(gt)
 
-    def summary(self) -> float:
-        """Computes the aggregated evaluation
+    def summary(self) -> Dict[str, float]:
+        """Computes the aggregated metrics
 
         Returns:
-            metric result"""
+            a dictionary with the exact match score for the raw data, its lower-case counterpart, its unidecode
+            counterpart and its lower-case unidecode counterpart
+        """
         if self.total == 0:
             raise AssertionError("you need to update the metric before getting the summary")
-        return self.matches / self.total
+
+        return dict(
+            raw=self.raw / self.total,
+            caseless=self.caseless / self.total,
+            unidecode=self.unidecode / self.total,
+            unicase=self.unicase / self.total,
+        )
 
     def reset(self) -> None:
-        self.matches = 0
+        self.raw = 0
+        self.caseless = 0
+        self.unidecode = 0
+        self.unicase = 0
         self.total = 0
 
 
@@ -138,21 +142,6 @@ def box_iou(boxes_1: np.ndarray, boxes_2: np.ndarray) -> np.ndarray:
     return iou_mat
 
 
-def assign_pairs(score_mat: np.ndarray, score_threshold: float = 0.5) -> Tuple[np.ndarray, np.ndarray]:
-    """Assigns candidates by maximizing the score of all pairs
-
-    Args:
-        score_mat: score matrix
-        score_threshold: minimum score to validate an assignment
-    Returns:
-        a tuple of two lists: the list of assigned row candidates indices, and the list of their column counterparts
-    """
-
-    row_ind, col_ind = linear_sum_assignment(-score_mat)
-    is_kept = score_mat[row_ind, col_ind] >= score_threshold
-    return row_ind[is_kept], col_ind[is_kept]
-
-
 class LocalizationConfusion:
     """Implements common confusion metrics and mean IoU for localization evaluation.
 
@@ -191,12 +180,8 @@ class LocalizationConfusion:
     """
 
     def __init__(self, iou_thresh: float = 0.5) -> None:
-
         self.iou_thresh = iou_thresh
-        self.num_gts = 0
-        self.num_preds = 0
-        self.num_matches = 0
-        self.tot_iou = 0.
+        self.reset()
 
     def update(self, gts: np.ndarray, preds: np.ndarray) -> None:
 
@@ -204,31 +189,37 @@ class LocalizationConfusion:
             # Compute IoU
             iou_mat = box_iou(gts, preds)
             self.tot_iou += float(iou_mat.max(axis=1).sum())
+
             # Assign pairs
-            gt_indices, _ = assign_pairs(iou_mat, self.iou_thresh)
-            self.num_matches += len(gt_indices)
+            gt_indices, pred_indices = linear_sum_assignment(-iou_mat)
+            self.matches += int((iou_mat[gt_indices, pred_indices] >= self.iou_thresh).sum())
 
         # Update counts
         self.num_gts += gts.shape[0]
         self.num_preds += preds.shape[0]
 
     def summary(self) -> Tuple[float, float, float]:
+        """Computes the aggregated metrics
+
+        Returns:
+            a tuple with the recall, precision and meanIoU scores
+        """
 
         # Recall
-        recall = self.num_matches / self.num_gts
+        recall = self.matches / self.num_gts if self.num_gts > 0 else None
 
         # Precision
-        precision = self.num_matches / self.num_preds
+        precision = self.matches / self.num_preds if self.num_preds > 0 else None
 
         # mean IoU
-        mean_iou = self.tot_iou / self.num_preds
+        mean_iou = self.tot_iou / self.num_preds if self.num_preds > 0 else None
 
         return recall, precision, mean_iou
 
     def reset(self) -> None:
         self.num_gts = 0
         self.num_preds = 0
-        self.num_matches = 0
+        self.matches = 0
         self.tot_iou = 0.
 
 
@@ -271,23 +262,11 @@ class OCRMetric:
 
     Args:
         iou_thresh: minimum IoU to consider a pair of prediction and ground truth as a match
-        max_dist: maximum Levenshtein distance between 2 sequence to consider a match
     """
 
-    def __init__(
-        self,
-        iou_thresh: float = 0.5,
-        max_dist: int = 0
-    ) -> None:
-
+    def __init__(self, iou_thresh: float = 0.5) -> None:
         self.iou_thresh = iou_thresh
-        self.max_dist = max_dist
-        self.num_gts = 0
-        self.num_preds = 0
-        self.num_det_matches = 0
-        self.num_reco_matches = 0
-        self.tot_iou = 0.
-        self.tot_dist = 0
+        self.reset()
 
     def update(
         self,
@@ -297,48 +276,62 @@ class OCRMetric:
         pred_labels: List[str],
     ) -> None:
 
+        if gt_boxes.shape[0] != len(gt_labels) or pred_boxes.shape[0] != len(pred_labels):
+            raise AssertionError("there should be the same number of boxes and string both for the ground truth "
+                                 "and the predictions")
+
         # Compute IoU
-        iou_mat = box_iou(gt_boxes, pred_boxes)
-        if iou_mat.shape[1] == 0:
-            self.tot_iou = 0
-        else:
+        if pred_boxes.shape[0] > 0:
+            iou_mat = box_iou(gt_boxes, pred_boxes)
             self.tot_iou += float(iou_mat.max(axis=1).sum())
 
-        # Assign pairs
-        gt_indices, preds_indices = assign_pairs(iou_mat, self.iou_thresh)
+            # Assign pairs
+            gt_indices, pred_indices = linear_sum_assignment(-iou_mat)
+            is_kept = iou_mat[gt_indices, pred_indices] >= self.iou_thresh
+            # String comparison
+            for gt_idx, pred_idx in zip(gt_indices[is_kept], pred_indices[is_kept]):
+                _raw, _caseless, _unidecode, _unicase = string_match(gt_labels[gt_idx], pred_labels[pred_idx])
+                self.raw_matches += int(_raw)
+                self.caseless_matches += int(_caseless)
+                self.unidecode_matches += int(_unidecode)
+                self.unicase_matches += int(_unicase)
 
-        # Compare sequences
-        for gt_idx, pred_idx in zip(gt_indices, preds_indices):
-            dist = levenshtein(gt_labels[gt_idx], pred_labels[pred_idx])
-            self.tot_dist += dist
-            if dist <= self.max_dist:
-                self.num_reco_matches += 1
-
-        # Update counts
-        self.num_det_matches = len(gt_indices)
         self.num_gts += gt_boxes.shape[0]
         self.num_preds += pred_boxes.shape[0]
 
-    def summary(self) -> Tuple[float, float, float, float]:
+    def summary(self) -> Tuple[Dict[str, float], Dict[str, float], float]:
+        """Computes the aggregated metrics
+
+        Returns:
+            a tuple with the recall & precision for each string comparison flexibility and the mean IoU
+        """
 
         # Recall
-        recall = self.num_reco_matches / self.num_gts
+        recall = dict(
+            raw=self.raw_matches / self.num_gts if self.num_gts > 0 else None,
+            caseless=self.caseless_matches / self.num_gts if self.num_gts > 0 else None,
+            unidecode=self.unidecode_matches / self.num_gts if self.num_gts > 0 else None,
+            unicase=self.unicase_matches / self.num_gts if self.num_gts > 0 else None,
+        )
 
         # Precision
-        precision = self.num_reco_matches / self.num_preds
+        precision = dict(
+            raw=self.raw_matches / self.num_preds if self.num_preds > 0 else None,
+            caseless=self.caseless_matches / self.num_preds if self.num_preds > 0 else None,
+            unidecode=self.unidecode_matches / self.num_preds if self.num_preds > 0 else None,
+            unicase=self.unicase_matches / self.num_preds if self.num_preds > 0 else None,
+        )
 
         # mean IoU (overall detected boxes)
-        mean_iou = self.tot_iou / self.num_preds
+        mean_iou = self.tot_iou / self.num_preds if self.num_preds > 0 else None
 
-        # mean distance (overall detection-matching boxes)
-        mean_distance = self.tot_dist / self.num_det_matches
-
-        return recall, precision, mean_iou, mean_distance
+        return recall, precision, mean_iou
 
     def reset(self) -> None:
         self.num_gts = 0
         self.num_preds = 0
-        self.num_det_matches = 0
-        self.num_reco_matches = 0
         self.tot_iou = 0.
-        self.tot_dist = 0
+        self.raw_matches = 0
+        self.caseless_matches = 0
+        self.unidecode_matches = 0
+        self.unicase_matches = 0
